﻿using System.Text;
using IDTechSDK;
using NLog.Fluent;




 void logMessage(string str)
{
    Console.WriteLine(str);

}

 ManualResetEvent connectDone;
bool connectionEvent = false;
bool didConnect = false;
string currentIdent = "";

 void MessageCallBack(IDTechSDK.IDT_DEVICE_Types type, DeviceState state, byte[] data,
IDTTransactionData cardData, EMV_Callback emvCallback, RETURN_CODE transactionResultCode, string ident)
{
    switch (state)
    {
        case DeviceState.TransactionData:
            break;
        case DeviceState.Connected:
            if (didConnect) return;
            Console.WriteLine("Connected : " + ident);
            currentIdent = ident;
            didConnect = true;
            connectionEvent = true;
            connectDone.Set();
            break;
        case DeviceState.ConnectionFailed:
            break;
        case DeviceState.DataReceived:
            break;
        case DeviceState.DataSent:
            break;
        case DeviceState.Disconnected:
            Console.WriteLine("Disconnected : " + ident);
            didConnect = false;
            connectionEvent = true;
            connectDone.Set();
            break;
        case DeviceState.SwipeCanceled:
            break;
        case DeviceState.ToConnect:
            break;
        case DeviceState.CommandTimeout:
            break;
        case DeviceState.SwipeTimeout:
            break;
        case DeviceState.DeviceTimeout:
            break;
        case DeviceState.EMVCallback:
            break;
        case DeviceState.TransactionCancelled:
            break;
        case DeviceState.TransactionFailed:
            break;
        case DeviceState.MSRDecodeError:
            break;
        case DeviceState.DefaultDeviceTypeChange:
            break;
        case DeviceState.Notification:
            break;
        case DeviceState.PINpadKeypress:
            break;
        case DeviceState.PINCancelled:
            break;
        case DeviceState.PINTimeout:
            break;
        case DeviceState.FirmwareUpdate:
            switch (transactionResultCode)
            {

                case RETURN_CODE.RETURN_CODE_FW_STARTING_UPDATE:
                    logMessage(" Starting Firmware Update");
                    break;
                case RETURN_CODE.RETURN_CODE_DO_SUCCESS:
                    logMessage(" Firmware Update Successful");
                    if (type == IDT_DEVICE_Types.IDT_DEVICE_VP8800) logMessage("The device will now reboot and validate the firmware file.  This process may take up to 5 minutes. You may proceed once device is recognized");

                    break;
                case RETURN_CODE.RETURN_CODE_APPLYING_FIRMWARE_UPDATE:
                    logMessage(" Applying Firmware Update....");
                    break;
                case RETURN_CODE.RETURN_CODE_ENTERING_BOOTLOADER_MODE:
                    logMessage(" Entering Bootloader Mode....");
                    break;
                case RETURN_CODE.RETURN_CODE_MULTI_FW_UPDATE_COMPLETE:
                    logMessage(" Multiple Firmware Update Sequence Complete");
                    break;
                case RETURN_CODE.RETURN_CODE_MULTI_FW_UPDATE_STARTING_NEXT:
                    logMessage(" Starting Next Firmware Update...");
                    break;
                case RETURN_CODE.RETURN_CODE_MULTI_FW_UPDATE_FAILED:
                    logMessage(" Multi Firmware Update process failed");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_PID_MISMATCH_ERROR:
                    logMessage("PID Mismatch in Manifest");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_SCRIPT_NOT_FOUND:
                    logMessage("Script Not Found in Manifest");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_ARCHIVE_UPDATE_IN_PROGRESS:
                    logMessage("Update From Archive In Process");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_ARCHIVE_UPDATE_STARTED:
                    logMessage("Update From Archive Started...");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_ARCHIVE_UPDATE_COMPLETED_WITH_SUCCESS:
                    logMessage("Update From Archive Completed Successfully");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_ARCHIVE_UPDATE_COMPLETED_WITH_FAILURE:
                    logMessage("Update From Archive Completed With Failure");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_COULD_NOT_RETRIEVE_DEVICE_INFO:
                    logMessage("Update From Archive Problem Retrieving Device Info");
                    break;
                case RETURN_CODE.RETURN_CODE_FW_FIRMWARE_ARCHIVE_MESSAGE:
                    if (data != null && data.Length > 0)
                    {
                        string msg = ASCIIEncoding.ASCII.GetString(data);
                        logMessage(msg + "");
                    }
                    break;
                case RETURN_CODE.RETURN_CODE_BLOCK_TRANSFER_SUCCESS:
                    int start = data[0];
                    int end = data[1];
                    if (data.Length > 3)
                    {
                        start = data[0] * 0x100 + data[1];
                        end = data[2] * 0x100 + data[3];
                    }
                    string label = "";
                    if (data.Length > 4) label = Common.FirmwareType((FIRMWARE_TYPE)data[4]);
                    logMessage(label + ": Sent block " + start.ToString() + " of " + end.ToString() + "");
                    break;
                case RETURN_CODE.RETURN_CODE_ERASING_SPI:
                    int start1 = data[0];
                    int end1 = data[1];
                    if (data.Length == 4)
                    {
                        start1 = data[0] * 0x100 + data[1];
                        end1 = data[2] * 0x100 + data[3];
                    }

                    logMessage(" Erasing SPI Flash: " + start1.ToString() + "% complete");
                    break;
                default:
                    logMessage("<ERROR>  Firmware Update Error Code: " + "0x" +
                                  String.Format("{0:X}", (ushort)transactionResultCode) + ": " +
                                  IDTechSDK.errorCode.getErrorString(transactionResultCode));
                    break;
            }

            break;
        case DeviceState.ConfigurationGroup:
            break;
        case DeviceState.CardAction:
            break;
        case DeviceState.Signature:
            break;
        case DeviceState.FileTransfer:
            break;
        case DeviceState.MenuItem:
            break;
        case DeviceState.MenuTimeout:
            break;
        case DeviceState.TransactionIdle:
            break;
        case DeviceState.VirtualData:
            break;
        case DeviceState.InvalidInterface:
            break;
        case DeviceState.FunctionKey:
            break;
        case DeviceState.PINFail:
            break;
        case DeviceState.SocketEstablished:
            break;
        case DeviceState.SocketFailed:
            break;
        case DeviceState.ButtonEvent:
            break;
        case DeviceState.LongPressEvent:
            break;
        case DeviceState.TenKeyEvent:
            break;
        case DeviceState.LogEvent:
            break;
        case DeviceState.RKI:
            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_STARTING_RKI_UPDATE)
            {
                logMessage(" Staring RKI Process.  Please Wait....");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_NO_SERIAL_NUMBER)
            {
                logMessage("<ERROR> RKI Error: Could not retrieve serial number");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_RETRIEVING_CERTIFICATES)
            {
                logMessage("<ERROR>  RKI Error: Could not retrieve certificates");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_RETRIEVING_KEY_DEVICE_GROUP)
            {
                logMessage("<ERROR>  RKI Error: Could not retrieve key device group");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_PEDI_FAILED)
            {
                logMessage("<ERROR>  RKI Error: PEDI Failed");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_INVALID_RSA)
            {
                logMessage("<ERROR>  RKI Error: Invalid RSA");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_PEDK_FAILED)
            {
                logMessage("<ERROR>  RKI Error: PEDK Failed");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_PEDK_SET_KEYS_FAILED)
            {
                logMessage("<ERROR>  RKI Error: PEDK Set Keys Failed");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_PEDV_FAILED)
            {
                logMessage("<ERROR>  RKI Error: PEDV Failed");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_PEDV_VERIFY_KEYS_FAILED)
            {
                logMessage("<ERROR>  RKI Error: PEDV Verify Keys Failed");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_COMPLETED_RKI_UPDATE_SUCCESS)
            {
                logMessage(" RKI Update Finished: Success");
            }

            if (transactionResultCode ==
                RETURN_CODE.RETURN_CODE_RKI_COMPLETED_RKI_UPDATE_SUCCESS_MORE_KEYS_AVAILABLE)
            {
                logMessage("  RKI Update Finished:  More Keys Availble. Run RKI again.");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_ENDING_SECURE_TASK)
            {
                logMessage("<ERROR>  RKI Error: Could not end secure task mode");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_UNKNOWN_FAILURE)
            {
                logMessage("<ERROR>  RKI Error: Unknown Failure");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_FAILED_DEVICE_KEY_REQUEST)
            {
                logMessage("<ERROR>  Failed on device key request, symmetric RKI.");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_FAILED_SERVER_RECEIVE_KEY)
            {
                logMessage("<ERROR>  Failed to get keys from the server, symmetric RKI.");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_FAILED_DEVICE_KEY_SET)
            {
                logMessage("<ERROR>  Failed on device to set key, symmetric RKI.");
            }

            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_ERROR_FAILED_SERVER_VALIDATE_KEY)
            {
                logMessage("<ERROR>  Failed to validate key with server, symmetric RKI.");
            }
            if (transactionResultCode == RETURN_CODE.RETURN_CODE_RKI_FAILURE)
            {
                logMessage("<ERROR>  RKI Failure.");
            }

            if (data != null && data.Length > 0)
                logMessage(ident + System.Text.Encoding.UTF8.GetString(data));
            break;
        case DeviceState.InputEvent:
            break;
        case DeviceState.AudioJackNotification:
            break;
        case DeviceState.CameraEventTimeout:
            break;
        case DeviceState.CameraEventData:
            break;
        case DeviceState.CTLSEvent:
            break;
        case DeviceState.CancelKeyPressed:
            break;
        case DeviceState.ViVOconfig:
            string outputtext = Encoding.UTF8.GetString(data, 0, data.Length);
            logMessage(outputtext);
            break;
        case DeviceState.BTListReady:
            break;
        case DeviceState.BTAlertReady:
            break;
        case DeviceState.PlugInserted:
            break;
        case DeviceState.PlugRemoved:
            break;
        case DeviceState.UniPayICC:
            break;
        case DeviceState.DeviceMessage:
            break;
        case DeviceState.RemoteMessage:
            break;
        case DeviceState.RemoteConnected:
            break;
        case DeviceState.RemoteDisconnected:
            break;
        case DeviceState.RemoteHostStarted:
            break;
        case DeviceState.RemoteHostEnded:
            break;
        case DeviceState.RemoteHeartbeat:
            break;
        case DeviceState.RemoteHostClosed:
            break;
        default:
            break;
    }
}

connectDone = new ManualResetEvent(false);
IDT_Device.setCallbackIP(MessageCallBack);
Profile.checkUSB();
IDT_Device.startUSBMonitoring();
if (Profile.connectionDict.Count > 0)
{
    Console.WriteLine("Please unplug all devices before starting test");
}
else
{
    Console.WriteLine("Testing Device Detection:  Please plug in device");
    connectDone.WaitOne();
    Console.WriteLine("DEVICE DETECTED");

    Console.WriteLine("Testing Device Removal:  Please unplug device");
    connectDone.Reset();
    connectDone.WaitOne();
    Console.WriteLine("DEVICE REMOVED");

    Console.WriteLine("Please plug in device");
    connectDone.Reset();
    connectDone.WaitOne();
    Console.WriteLine("DEVICE DETECTED");
    Console.WriteLine("Testing Device Reboot: No disconnect/connect events should happen");
    connectDone.Reset();
    TimeSpan ts = new TimeSpan(0, 0, 3);
    connectDone.WaitOne(ts);
    connectionEvent = false;
    connectDone.Reset();
    didConnect = false;
    RETURN_CODE rt = IDT_Device.SharedController.device_rebootDevice(currentIdent);
    connectDone.WaitOne(ts);
    if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS && !connectionEvent)
    {
        Console.WriteLine("Device Rebooted Successfully without connection events");
        Console.WriteLine("Testing communication after reboot");
        string fw = null;
        rt = IDT_Device.SharedController.device_getFirmwareVersion(ref fw,currentIdent);
        if (rt != RETURN_CODE.RETURN_CODE_DO_SUCCESS)
        {
            if (connectionEvent)
            {
                Console.WriteLine("Getting FW version failed. Connection events fired from previous reboot. Communication Bad");

            }
            else
            {
                Console.WriteLine("Getting FW version failed. Communication Bad");

            }
        }
        else
        {
            Console.WriteLine("Getting FW version success. Communication Good");
            Console.WriteLine(fw);
            Console.WriteLine("All Tests Passed");

        }
    }
    else
    {
        if (rt != RETURN_CODE.RETURN_CODE_DO_SUCCESS)
        {
            Console.WriteLine("Reboot Attempt Failed");
        }
        if (connectionEvent)
        {
            Console.WriteLine("Reboot successful, but connect/disconnect events fired");
        }
    }


}
Console.WriteLine("Application Finished Running");



